package com.common.commonlib.image.loader

import android.content.Context
import android.widget.ImageView
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.load.resource.drawable.DrawableTransitionOptions
import com.common.commonlib.image.config.GlideBaseConfig
import com.common.commonlib.image.config.GlideHolderConfig
import com.common.commonlib.image.config.GlideTransitionConfig
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking

/**
 * 图片加载类
 */
object ImageLoader : ILoader {

    override fun init(
        context: Context, memoryCategory: GlideBaseConfig.MemoryType
    ) {
        Glide.get(context).setMemoryCategory(memoryCategory.memoryCategory)
    }

    fun setGlobalConfig(globalHolderConfig: GlideHolderConfig) {
        GlobalConfigHelper.globalHolderConfig = globalHolderConfig
    }

    fun setTransition(transitionConfig: GlideTransitionConfig) {
        GlobalConfigHelper.globalTransition = transitionConfig.transition
    }

    override fun load(context: Context, url: String?, view: ImageView) {
        loadWithMode(context, url, view, GlideBaseConfig.LoadMode.DEFAULT)
    }

    override fun loadWithMode(
        context: Context,
        url: String?,
        view: ImageView,
        loadingMode: GlideBaseConfig.LoadMode
    ) {
        val request = GlobalConfigHelper.loadWithGlobalConfig(Glide.with(context).load(url))
        when (loadingMode) {
            GlideBaseConfig.LoadMode.DEFAULT ->
                request.into(view)
            GlideBaseConfig.LoadMode.ALL ->
                request.diskCacheStrategy(DiskCacheStrategy.ALL).into(view)
            GlideBaseConfig.LoadMode.SKIP_CACHE ->
                request.skipMemoryCache(true)
                    .diskCacheStrategy(DiskCacheStrategy.NONE)
                    .into(view)
            GlideBaseConfig.LoadMode.SKIP_DISK ->
                request.diskCacheStrategy(DiskCacheStrategy.NONE).into(view)
            GlideBaseConfig.LoadMode.SKIP_MEM ->
                request.skipMemoryCache(true).into(view)
            GlideBaseConfig.LoadMode.SKIP_NET ->
                request.onlyRetrieveFromCache(true).into(view)
        }
    }

    override fun loadWithPlaceHolder(
        context: Context,
        url: String,
        view: ImageView,
        placeGlideHolder: GlideHolderConfig
    ) {
        GlobalConfigHelper.loadWithGlobalConfig(Glide.with(context).load(url), placeGlideHolder)
            .into(view)
    }

    override fun clearMemoryCache(context: Context) = runBlocking<Unit> {
        launch(Dispatchers.Main) {
            Glide.get(context).clearMemory()
        }
    }

    override fun clearDiskCache(context: Context) = runBlocking<Unit> {
        launch(Dispatchers.IO) {
            Glide.get(context).clearDiskCache()
        }
    }

    override fun clearAllCache(context: Context) = runBlocking<Unit> {
        launch(Dispatchers.Main) {
            Glide.get(context).clearMemory()
        }
        launch(Dispatchers.IO) {
            Glide.get(context).clearDiskCache()
        }
    }
}